@@ -124,13 +124,18 @@ struct Op {
   // Miller is defined in the same way in namespace gemmi in unitcell.hpp
   using Miller = std::array<int, 3>;
 
-  Miller apply_to_hkl(const Miller& hkl) const {
+  Miller apply_to_hkl_without_division(const Miller& hkl) const {
     Miller r;
     for (int i = 0; i != 3; ++i)
-      r[i] = (rot[0][i] * hkl[0] + rot[1][i] * hkl[1] + rot[2][i] * hkl[2])
-             / Op::DEN;
+      r[i] = (rot[0][i] * hkl[0] + rot[1][i] * hkl[1] + rot[2][i] * hkl[2]);
     return r;
   }
+  static Miller divide_hkl_by_DEN(const Miller& hkl) {
+    return {{ hkl[0] / DEN, hkl[1] / DEN, hkl[2] / DEN }};
+  }
+  Miller apply_to_hkl(const Miller& hkl) const {
+    return divide_hkl_by_DEN(apply_to_hkl_without_division(hkl));
+  }
 
   double phase_shift(const Miller& hkl) const {
     constexpr double mult = -2 * 3.1415926535897932384626433832795 / Op::DEN;
@@ -388,6 +393,43 @@ struct GroupOps {
     return find_by_rotation({-Op::DEN,0,0, 0,-Op::DEN,0, 0,0,-Op::DEN}) != nullptr;
   }
 
+  bool is_reflection_centric(const Op::Miller& hkl) const {
+    Op::Miller mhkl = {{-Op::DEN * hkl[0], -Op::DEN * hkl[1], -Op::DEN * hkl[2]}};
+    for (const Op& op : sym_ops)
+      if (op.apply_to_hkl_without_division(hkl) == mhkl)
+        return true;
+    return false;
+  }
+
+  int epsilon_factor(const Op::Miller& hkl) const {
+    Op::Miller denh = {{Op::DEN * hkl[0], Op::DEN * hkl[1], Op::DEN * hkl[2]}};
+    int epsilon = 0;
+    for (const Op& op : sym_ops)
+      if (op.apply_to_hkl_without_division(hkl) == denh)
+        ++epsilon;
+    return epsilon;
+  }
+
+  static bool has_phase_shift(const Op::Tran& c, const Op::Miller& hkl) {
+    return (hkl[0] * c[0] + hkl[1] * c[1] + hkl[2] * c[2]) % Op::DEN != 0;
+  }
+
+  bool is_systematically_absent(const Op::Miller& hkl) const {
+    for (auto i = cen_ops.begin() + 1; i != cen_ops.end(); ++i)
+      if (has_phase_shift(*i, hkl))
+        return true;
+    Op::Miller denh = {{Op::DEN * hkl[0], Op::DEN * hkl[1], Op::DEN * hkl[2]}};
+    for (auto op = sym_ops.begin() + 1; op != sym_ops.end(); ++op)
+      if (op->apply_to_hkl_without_division(hkl) == denh) {
+        for (const Op::Tran& c : cen_ops)
+          if (has_phase_shift({{op->tran[0] + c[0],
+                                op->tran[1] + c[1],
+                                op->tran[2] + c[2]}}, hkl))
+            return true;
+      }
+    return false;
+  }
+
   void change_basis(const Op& cob) {
     if (sym_ops.empty() || cen_ops.empty())
       return;
@@ -1729,11 +1771,11 @@ const SpaceGroup* find_spacegroup_by_change_of_basis(const SpaceGroup* sg,
 
 // Reciprocal space asu (asymmetric unit).
 // The same 12 choices of ASU as in CCP4 symlib and cctbx.
-struct ReciprocalAsuChecker {
+struct ReciprocalAsu {
   int idx;
   Op::Rot rot;
 
-  ReciprocalAsuChecker(const SpaceGroup* sg) {
+  ReciprocalAsu(const SpaceGroup* sg) {
     if (sg == nullptr)
       fail("Missing space group");
     rot = sg->basisop().rot;
@@ -1778,6 +1820,18 @@ struct ReciprocalAsuChecker {
     }
     unreachable();
   }
+
+  Op::Miller to_asu(Op::Miller hkl, const GroupOps& gops) const {
+    for (const Op& op : gops.sym_ops) {
+      Op::Miller new_hkl = op.apply_to_hkl_without_division(hkl);
+      if (is_in(new_hkl))
+        return Op::divide_hkl_by_DEN(new_hkl);
+      Op::Miller negated_new_hkl{{-new_hkl[0], -new_hkl[1], -new_hkl[2]}};
+      if (is_in(negated_new_hkl))
+        return Op::divide_hkl_by_DEN(negated_new_hkl);
+    }
+    fail("Oops, maybe inconsistent GroupOps?");
+  }
 };
 
 } // namespace gemmi
