@@ -17,6 +17,7 @@ struct PdbWriteOptions {
   bool cispep_records = true;
   bool ter_records = true;
   bool numbered_ter = true;
+  bool ter_ignores_type = false;
   bool use_linkr = false;
 };
 
@@ -149,7 +150,7 @@ inline void write_multiline(std::ostream& os, const char* record_name,
   for (int n = 2; n < 1000 && *end != '\0'; ++n) {
     start = end;
     end = find_last_break(start, lastcol-11);
-    int len = end - start;
+    int len = int(end - start);
     WRITEU("%-6s %3d %-69.*s\n", record_name, n, len, start);
   }
 }
@@ -324,9 +325,11 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
         os.write(buf, 81);
       }
     }
-    if (opt.ter_records && res.entity_type == EntityType::Polymer &&
-        (&res == &chain.residues.back() ||
-         (&res + 1)->entity_type != EntityType::Polymer)) {
+    if (opt.ter_records &&
+        (opt.ter_ignores_type ? &res == &chain.residues.back()
+                              : (res.entity_type == EntityType::Polymer &&
+                                (&res == &chain.residues.back() ||
+                                 (&res + 1)->entity_type != EntityType::Polymer)))) {
       if (opt.numbered_ter) {
         // re-using part of the buffer in the middle, e.g.:
         // TER    4153      LYS B 286
